# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: persist.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rpersist.proto\x12\x07persist\";\n\x12\x46\x65\x61tureGroupSchema\x12\r\n\x05label\x18\x01 \x01(\t\x12\x16\n\x0e\x66\x65\x61ture_labels\x18\x02 \x03(\t\"J\n\x04\x44\x61ta\x12\x12\n\nkey_values\x18\x01 \x03(\t\x12.\n\x0e\x66\x65\x61ture_values\x18\x02 \x03(\x0b\x32\x16.persist.FeatureValues\"0\n\rFeatureValues\x12\x1f\n\x06values\x18\x01 \x01(\x0b\x32\x0f.persist.Values\"\xd9\x01\n\x06Values\x12\x13\n\x0b\x66p32_values\x18\x01 \x03(\x01\x12\x13\n\x0b\x66p64_values\x18\x02 \x03(\x01\x12\x14\n\x0cint32_values\x18\x03 \x03(\x05\x12\x14\n\x0cint64_values\x18\x04 \x03(\x03\x12\x15\n\ruint32_values\x18\x05 \x03(\r\x12\x15\n\ruint64_values\x18\x06 \x03(\x04\x12\x15\n\rstring_values\x18\x07 \x03(\t\x12\x13\n\x0b\x62ool_values\x18\x08 \x03(\x08\x12\x1f\n\x06vector\x18\t \x03(\x0b\x32\x0f.persist.Vector\")\n\x06Vector\x12\x1f\n\x06values\x18\x01 \x01(\x0b\x32\x0f.persist.Values\"\x8a\x01\n\x05Query\x12\x14\n\x0c\x65ntity_label\x18\x01 \x01(\t\x12\x13\n\x0bkeys_schema\x18\x02 \x03(\t\x12\x39\n\x14\x66\x65\x61ture_group_schema\x18\x03 \x03(\x0b\x32\x1b.persist.FeatureGroupSchema\x12\x1b\n\x04\x64\x61ta\x18\x04 \x03(\x0b\x32\r.persist.Data\"\x19\n\x06Result\x12\x0f\n\x07message\x18\x02 \x01(\t2F\n\x0e\x46\x65\x61tureService\x12\x34\n\x0fPersistFeatures\x12\x0e.persist.Query\x1a\x0f.persist.Result\"\x00\x42\x0cZ\n../persistb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'persist_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\n../persist'
  _FEATUREGROUPSCHEMA._serialized_start=26
  _FEATUREGROUPSCHEMA._serialized_end=85
  _DATA._serialized_start=87
  _DATA._serialized_end=161
  _FEATUREVALUES._serialized_start=163
  _FEATUREVALUES._serialized_end=211
  _VALUES._serialized_start=214
  _VALUES._serialized_end=431
  _VECTOR._serialized_start=433
  _VECTOR._serialized_end=474
  _QUERY._serialized_start=477
  _QUERY._serialized_end=615
  _RESULT._serialized_start=617
  _RESULT._serialized_end=642
  _FEATURESERVICE._serialized_start=644
  _FEATURESERVICE._serialized_end=714
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: retrieve.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0eretrieve.proto\x12\x08retrieve\"\x14\n\x04Keys\x12\x0c\n\x04\x63ols\x18\x01 \x03(\t\"\x80\x01\n\x05Query\x12\x14\n\x0c\x65ntity_label\x18\x01 \x01(\t\x12.\n\x0e\x66\x65\x61ture_groups\x18\x02 \x03(\x0b\x32\x16.retrieve.FeatureGroup\x12\x13\n\x0bkeys_schema\x18\x03 \x03(\t\x12\x1c\n\x04keys\x18\x04 \x03(\x0b\x32\x0e.retrieve.Keys\",\n\x07\x46\x65\x61ture\x12\r\n\x05label\x18\x01 \x01(\t\x12\x12\n\ncolumn_idx\x18\x02 \x01(\x05\"Q\n\rFeatureSchema\x12\x1b\n\x13\x66\x65\x61ture_group_label\x18\x01 \x01(\t\x12#\n\x08\x66\x65\x61tures\x18\x02 \x03(\x0b\x32\x11.retrieve.Feature\"$\n\x03Row\x12\x0c\n\x04keys\x18\x01 \x03(\t\x12\x0f\n\x07\x63olumns\x18\x02 \x03(\x0c\"\x82\x01\n\x06Result\x12\x14\n\x0c\x65ntity_label\x18\x01 \x01(\t\x12\x13\n\x0bkeys_schema\x18\x02 \x03(\t\x12\x30\n\x0f\x66\x65\x61ture_schemas\x18\x03 \x03(\x0b\x32\x17.retrieve.FeatureSchema\x12\x1b\n\x04rows\x18\x04 \x03(\x0b\x32\r.retrieve.Row\"z\n\rDecodedResult\x12\x13\n\x0bkeys_schema\x18\x01 \x03(\t\x12\x30\n\x0f\x66\x65\x61ture_schemas\x18\x02 \x03(\x0b\x32\x17.retrieve.FeatureSchema\x12\"\n\x04rows\x18\x03 \x03(\x0b\x32\x14.retrieve.DecodedRow\"+\n\nDecodedRow\x12\x0c\n\x04keys\x18\x01 \x03(\t\x12\x0f\n\x07\x63olumns\x18\x02 \x03(\t\"5\n\x0c\x46\x65\x61tureGroup\x12\r\n\x05label\x18\x01 \x01(\t\x12\x16\n\x0e\x66\x65\x61ture_labels\x18\x02 \x03(\t2\x8e\x01\n\x0e\x46\x65\x61tureService\x12\x37\n\x10RetrieveFeatures\x12\x0f.retrieve.Query\x1a\x10.retrieve.Result\"\x00\x12\x43\n\x15RetrieveDecodedResult\x12\x0f.retrieve.Query\x1a\x17.retrieve.DecodedResult\"\x00\x42\rZ\x0b../retrieveb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'retrieve_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\013../retrieve'
  _KEYS._serialized_start=28
  _KEYS._serialized_end=48
  _QUERY._serialized_start=51
  _QUERY._serialized_end=179
  _FEATURE._serialized_start=181
  _FEATURE._serialized_end=225
  _FEATURESCHEMA._serialized_start=227
  _FEATURESCHEMA._serialized_end=308
  _ROW._serialized_start=310
  _ROW._serialized_end=346
  _RESULT._serialized_start=349
  _RESULT._serialized_end=479
  _DECODEDRESULT._serialized_start=481
  _DECODEDRESULT._serialized_end=603
  _DECODEDROW._serialized_start=605
  _DECODEDROW._serialized_end=648
  _FEATUREGROUP._serialized_start=650
  _FEATUREGROUP._serialized_end=703
  _FEATURESERVICE._serialized_start=706
  _FEATURESERVICE._serialized_end=848
# @@protoc_insertion_point(module_scope)



import re
import sys
import requests

def clean_column_name(column_name):
    """
    Cleans a column name to be Delta Table compatible:
    - Replaces unsupported special characters with "_"
    - Ensures it doesn't start with a number
    - Converts to lowercase
    """
    # Define a regex pattern to match invalid characters
    invalid_chars = r"[^a-zA-Z0-9_]"  
    cleaned_name = re.sub(invalid_chars, "_", column_name)  # Replace invalid chars with "_"

    # Ensure it doesn't start with a number
    if cleaned_name[0].isdigit():
        cleaned_name = "_" + cleaned_name

    return cleaned_name.lower()  # Convert to lowercase (Delta standard)

def get_metadata_host_response(features_metadata_url: str, job_id: str, job_token: str) -> dict:
    
    assert features_metadata_url.startswith("http") or features_metadata_url.startswith("https"), "features_metadata_url must start with http or https"
    
    params = {
        "jobId": job_id,
        "jobToken": job_token
    }
    headers = {
        "Content-Type": "application/json"
    }

    response = requests.post(features_metadata_url, headers=headers, params=params)

    assert response.status_code == 200, f"Request failed with status code {response.status_code}: {response.json()} while calling {features_metadata_url} for job_id: {job_id} and job_token: {job_token}"

    return response.json()

def get_features_from_response_v2(json_response, fg_to_consider):
    entity_label = list(json_response["keys"].keys())[0]
    entity_column_names = list(json_response["keys"].values())[0]
    
    skip_check = True if not fg_to_consider else False
    fg_to_datatype_map = {}

    responses = json_response["data"]
    feature_list = []
    offline_col_to_default_values_map = {}
    offline_col_to_datatype_map = {}
    feature_kv = {}
    for sources in responses:

        source_type = sources["storage-provider"]

        base_path_config = sources['base-path']


        for table in base_path_config:

            table_name = table["source-base-path"]
            

            features_list_temp = []
            for features in table["data-paths"]:
                fg_label = features["feature-group-label"]
                fg_data_type = features["data-type"]
                
                if fg_label not in fg_to_datatype_map:
                    fg_to_datatype_map[fg_label] = fg_data_type

                if not skip_check and fg_label not in fg_to_consider:
                    continue

                feature_col = features["source-data-column"]
                feature_label = features["feature-label"]
                
                # create rename feature col as there can be same column from multiple sources and same source can be used in multiple feature groups
                if source_type == "TABLE":
                    rename_feature_col = table_name.split(".")[1] + "___" + fg_label + "___" + feature_label
                                        
                elif source_type in ["PARQUET_GCS", # GCP
                                     "PARQUET_S3", # AWS
                                     "PARQUET_ADLS", # Azure
                                     "DELTA_GCS", # GCP
                                     "DELTA_S3", # AWS
                                     "DELTA_ADLS" # Azure
                                     ]:
                    rename_feature_col = clean_column_name(table_name.split("gs://")[1].strip("/ ").split("/")[-1]) + "___" + fg_label + "___" + feature_label
                    
                else:
                    print(f"source: {table_name} of type {source_type} not expected")
                    sys.exit(1)
                
                feature_default = features["default-value"]

                features_list_temp.append((feature_col, rename_feature_col))

                offline_col_to_default_values_map[rename_feature_col] = feature_default
                offline_col_to_datatype_map[rename_feature_col] = fg_data_type


                if fg_label not in feature_kv:
                    feature_kv[fg_label] = [(rename_feature_col, feature_label)]

                else:
                    feature_kv[fg_label].append((rename_feature_col, feature_label))

            if len(features_list_temp) != 0:
                feature_list.append([table_name, source_type, features_list_temp])
                
            onfs_fg_to_onfs_feat_map, onfs_fg_to_ofs_feat_map = get_fgs_to_feature_mappings(feature_kv)
    return feature_list, offline_col_to_default_values_map, onfs_fg_to_onfs_feat_map, onfs_fg_to_ofs_feat_map, fg_to_datatype_map, entity_label, entity_column_names, offline_col_to_datatype_map

def get_features_details(features_metadata_url: str, job_id: str, job_token: str, fgs_to_consider: list = []) -> tuple:
    
    custodian_response = get_metadata_host_response(features_metadata_url, job_id, job_token)
    return get_features_from_response_v2(custodian_response, fgs_to_consider)

def get_fgs_to_feature_mappings(feature_group_kv):

    onfs_fg_to_onfs_feat_map = {} # onfs fg -> online feat name
    for fg in feature_group_kv:
        if fg not in onfs_fg_to_onfs_feat_map:
            onfs_fg_to_onfs_feat_map[fg] = []

        for onfs_feat_name in feature_group_kv[fg]:
            onfs_fg_to_onfs_feat_map[fg].append(onfs_feat_name[1])

    onfs_fg_to_ofs_feat_map = {} # onfs fg -> offline feat name

    for fg in feature_group_kv:
        if fg not in onfs_fg_to_ofs_feat_map:
            onfs_fg_to_ofs_feat_map[fg] = []

        for onfs_feat_name in feature_group_kv[fg]:
            onfs_fg_to_ofs_feat_map[fg].append(onfs_feat_name[0])       
            
    return onfs_fg_to_onfs_feat_map, onfs_fg_to_ofs_feat_map